package org.bitbucket.bradleysmithllc.etlunit.feature.workspace;

import org.bitbucket.bradleysmithllc.etlunit.*;
import org.bitbucket.bradleysmithllc.etlunit.context.VariableContext;
import org.bitbucket.bradleysmithllc.etlunit.feature.AbstractFeature;
import org.bitbucket.bradleysmithllc.etlunit.feature.RuntimeOption;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestMethod;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestValueObjectImpl;

import javax.inject.Inject;
import javax.inject.Named;
import java.io.IOException;

/**
 * The purpose of this class is to (1) provide a workspace for temporary
 * files, etc, which will get cleared with each test.  It will also (2)
 * expose an option to purge the workspace between tests.
 */
public class WorkspaceFeatureModule extends AbstractFeature {
	private RuntimeSupport runtimeSupport;
	private boolean purgeWorkspace = true;

	@Inject
	public void setpurgeWorkspace(@Named("workspace.purgeWorkspace") RuntimeOption option)
	{
		purgeWorkspace = option.isEnabled();
	}

	@Inject
	public void receiveRuntimeSupport(RuntimeSupport rs) {
		runtimeSupport = rs;
	}

	@Override
	public ClassListener getListener() {
		return new NullClassListener() {
			@Override
			public void begin(ETLTestMethod mt, VariableContext context) throws TestAssertionFailure, TestExecutionError, TestWarning {
				context = context.getTopLevelScope();

				try {
					// check for a declared workspace
					Workspace workspace = null;

					if (!context.hasVariableBeenDeclared("workspace")) {
						// create and install a new one
						workspace = new Workspace(runtimeSupport.createTempFile("workspace-" + System.currentTimeMillis()));

						context.declareAndSetValue("workspace", new ETLTestValueObjectImpl(workspace));
					} else {
						workspace = (Workspace) context.getValue("workspace").getValueAsPojo();
					}

					// optionally purge
					if (purgeWorkspace)
					{
						workspace.purge();
					}
				} catch (IOException exc) {
					throw new TestExecutionError("", TestConstants.ERR_IO_ERROR, exc);
				}
			}
		};
	}
}