package org.bitbucket.bradleysmithllc.etlunit.metadata.impl;

import com.fasterxml.jackson.databind.JsonNode;
import com.google.gson.stream.JsonWriter;
import org.apache.commons.lang3.ObjectUtils;
import org.bitbucket.bradleysmithllc.etlunit.metadata.TestReference;

import java.io.IOException;

public class TestReferenceImpl implements TestReference
{
	private String classification;
	private String referencingPackage;
	private String referencingTestClass;
	private String referencingTestName;
	private String qualifiedReferencingTestName;

	public TestReferenceImpl() {
	}

	public TestReferenceImpl(String classification, String referencingPackage, String referencingTestClass, String referencingTestName, String qName) {
		this.classification = classification;

		if (referencingPackage != null && !referencingPackage.equals(""))
		{
			this.referencingPackage = referencingPackage;
		}

		this.referencingTestClass = referencingTestClass;
		this.referencingTestName = referencingTestName;
		qualifiedReferencingTestName = qName;
	}

	@Override
	public String getClassification() {
		return classification;
	}

	@Override
	public String getReferencingTestPackage() {
		return referencingPackage;
	}

	@Override
	public String getReferencingTestClass() {
		return referencingTestClass;
	}

	@Override
	public String getReferencingTestName() {
		return referencingTestName;
	}

	@Override
	public String getQualifiedTestName() {
		return qualifiedReferencingTestName;
	}

	@Override
	public void toJson(JsonWriter writer) throws IOException {
		writer.name("classification").value(classification);

		writer.name("package");

		if (referencingPackage == null)
		{
			writer.nullValue();
		}
		else
		{
			writer.value(referencingPackage);
		}
		writer.name("class").value(referencingTestClass);
		writer.name("method").value(referencingTestName);

		writer.name("qualified-test-name").value(qualifiedReferencingTestName);
	}

	@Override
	public void fromJson(JsonNode node) {
		classification = node.get("classification").asText();
		referencingTestName = node.get("method").asText();
		referencingTestClass = node.get("class").asText();
		qualifiedReferencingTestName = node.get("qualified-test-name").asText();

		JsonNode aPackage = node.get("package");
		if (aPackage != null && !aPackage.isNull())
		{
			referencingPackage = aPackage.asText();
		}
	}

	public boolean equals(Object reference)
	{
		if (reference.getClass() != getClass())
		{
			return false;
		}

		TestReferenceImpl oth = (TestReferenceImpl) reference;

		return
			ObjectUtils.compare(classification, oth.classification) == 0 &&
			ObjectUtils.compare(qualifiedReferencingTestName, oth.qualifiedReferencingTestName) == 0 &&
			ObjectUtils.compare(referencingPackage, oth.referencingPackage) == 0 &&
			ObjectUtils.compare(referencingTestClass, oth.referencingTestClass) == 0 &&
			ObjectUtils.compare(referencingTestName, oth.referencingTestName) == 0;
	}
}